﻿using System;
using System.Windows.Forms;

using MissionPlanner;
using MissionPlanner.Utilities;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using MissionPlanner.Controls;

using GMap.NET;
using GMap.NET.WindowsForms;
using System.Drawing.Drawing2D;

namespace RegenarateErrorPlugins
{
    public class RegenerateErrorPlugins : MissionPlanner.Plugin.Plugin
    {
        private AnchorageTextOverlay AnchorageTextOverlayFD;
        private AnchorageTextOverlay AnchorageTextOverlayFP;

        public override string Name
        {
            get { return "Regenerate Error Plugins"; }
        }

        public override string Version
        {
            get { return "0.30"; }
        }

        public override string Author
        {
            get { return "Open Source Community"; }
        }

        public override bool Init()
        {
            return true;
        }

        public override bool Loaded()
        {
            //1. Adding the menu button items.
            var rootbut_FD = new ToolStripMenuItem("Regenerate Error Plugins");
            ToolStripItemCollection col_FD = Host.FDMenuMap.Items; 
            col_FD.Add(rootbut_FD);

            var but = new ToolStripMenuItem("Start/Stop Displaying Anchorage Text Overlay on FD");
            but.Click += DisplayingAnchorageTextOverlay_FD_Click;
            rootbut_FD.DropDownItems.Add(but);

            //2. Adding the overlays for displaying text and polygons on the map.
            AnchorageTextOverlayFD = new AnchorageTextOverlay(Host.FDGMapControl);//gave parameter FDGMapControl object for possible future use. but I'm not sure it is right way to programming.

            AnchorageTextOverlayFD.IsVisibile = true;

            Host.FDGMapControl.Overlays.Add(AnchorageTextOverlayFD);

            MarkingAnchorageArea(); //Draw polygons on the map.

            //3. loop rate should be fast when I get anchored ship's datas
            loopratehz = 12.0f;

            return true;
        }

        public override bool Loop()
        {
            loopratehz = 20.0f; //loop rate should be fast when I get anchored ship's datas
            return true;
        }

        public override bool Exit()
        {
            return true;
        }

        //Define the function executed when clicks the button.(Event handler)
        private void DisplayingAnchorageTextOverlay_FD_Click(object sender, EventArgs e)
        {
            if (AnchorageTextOverlayFD.IsVisibile)
            {
                AnchorageTextOverlayFD.IsVisibile = false;
                AnchorageTextOverlayFD.IsVisibile = false;
                MessageBox.Show("Stopped displaying anchorage area text", "Display anchorage area options");
            }
            else
            {
                AnchorageTextOverlayFD.IsVisibile = true;
                AnchorageTextOverlayFD.IsVisibile = true;
                MessageBox.Show("Started displaying anchorage area text", "Display anchorage area options");
            }
        }

        //Draw Color-filled-Polygons on Ship's Anchorage area
        private void MarkingAnchorageArea()
        {
            var points_N1 = new List<PointLatLng>();
            points_N1.Add(new PointLatLng(35.07641667, 129.0333333));
            points_N1.Add(new PointLatLng(35.07641667, 129.0393056));
            points_N1.Add(new PointLatLng(35.06713889, 129.0530833));
            points_N1.Add(new PointLatLng(35.06713889, 129.0344167));

            var points_N2 = new List<PointLatLng>();
            points_N2.Add(new PointLatLng(35.07644444, 129.0265278));
            points_N2.Add(new PointLatLng(35.07644444, 129.0315));
            points_N2.Add(new PointLatLng(35.053, 129.0312222));
            points_N2.Add(new PointLatLng(35.05294444, 129.0172778));
            points_N2.Add(new PointLatLng(35.06147222, 129.0249167));
            points_N2.Add(new PointLatLng(35.07413889, 129.0249167));

            var points_N3 = new List<PointLatLng>();
            points_N3.Add(new PointLatLng(35.06713889, 129.0344167));
            points_N3.Add(new PointLatLng(35.06713889, 129.0530833));
            points_N3.Add(new PointLatLng(35.04886111, 129.0800556));
            points_N3.Add(new PointLatLng(35.04430556, 129.0370833));

            var points_N4 = new List<PointLatLng>();
            points_N4.Add(new PointLatLng(35.04430556, 129.0370833));
            points_N4.Add(new PointLatLng(35.04886111, 129.0800556));
            points_N4.Add(new PointLatLng(35.036, 129.0899167));
            points_N4.Add(new PointLatLng(35.02686111, 129.0439167));

            var points_N5 = new List<PointLatLng>();
            points_N5.Add(new PointLatLng(35.02686111, 129.0439167));
            points_N5.Add(new PointLatLng(35.036, 129.0899167));
            points_N5.Add(new PointLatLng(35.00313889, 129.0800278));
            points_N5.Add(new PointLatLng(35.00319444, 129.0439167));

            var points_O2 = new List<PointLatLng>();
            points_O2.Add(new PointLatLng(35.09611111, 129.0683333));
            points_O2.Add(new PointLatLng(35.08944444, 129.075));
            points_O2.Add(new PointLatLng(35.09444444, 129.0805556));
            points_O2.Add(new PointLatLng(35.1, 129.0713889));

            var polygon_N1 = new GMapPolygon(points_N1, "Polygon");
            var polygon_N2 = new GMapPolygon(points_N2, "Polygon");
            var polygon_N3 = new GMapPolygon(points_N3, "Polygon");
            var polygon_N4 = new GMapPolygon(points_N4, "Polygon");
            var polygon_N5 = new GMapPolygon(points_N5, "Polygon");
            var polygon_O2 = new GMapPolygon(points_O2, "Polygon");

            polygon_N1.Fill = new SolidBrush(Color.FromArgb(50, Color.Blue));
            polygon_N1.Stroke = new Pen(Color.Blue, 2);
            polygon_N2.Fill = new SolidBrush(Color.FromArgb(50, Color.Blue));
            polygon_N2.Stroke = new Pen(Color.Blue, 2);
            polygon_N3.Fill = new SolidBrush(Color.FromArgb(50, Color.Blue));
            polygon_N3.Stroke = new Pen(Color.Blue, 2);
            polygon_N4.Fill = new SolidBrush(Color.FromArgb(50, Color.Blue));
            polygon_N4.Stroke = new Pen(Color.Blue, 2);
            polygon_N5.Fill = new SolidBrush(Color.FromArgb(50, Color.Blue));
            polygon_N5.Stroke = new Pen(Color.Blue, 2);
            polygon_O2.Fill = new SolidBrush(Color.FromArgb(50, Color.Blue));
            polygon_O2.Stroke = new Pen(Color.Blue, 2);

            AnchorageTextOverlayFD.Polygons.Add(polygon_N1);
            AnchorageTextOverlayFD.Polygons.Add(polygon_N2);
            AnchorageTextOverlayFD.Polygons.Add(polygon_N3);
            AnchorageTextOverlayFD.Polygons.Add(polygon_N4);
            AnchorageTextOverlayFD.Polygons.Add(polygon_N5);
            AnchorageTextOverlayFD.Polygons.Add(polygon_O2);
            AnchorageTextOverlayFD.Polygons.Add(polygon_N1);
            AnchorageTextOverlayFD.Polygons.Add(polygon_N2);
            AnchorageTextOverlayFD.Polygons.Add(polygon_N3);
            AnchorageTextOverlayFD.Polygons.Add(polygon_N4);
            AnchorageTextOverlayFD.Polygons.Add(polygon_N5);
            AnchorageTextOverlayFD.Polygons.Add(polygon_O2);
        }
    }//end of Class Regenerate Error Plugins

    //I inheritted GMapOverlay class to override rendering logic to overlay text on certain map's coordinates
    public class AnchorageTextOverlay : GMapOverlay
    {
        //Latitude and Longitude Coordinates.
        private PointLatLng N1LatLng;
        private PointLatLng N2LatLng;
        private PointLatLng N3LatLng;
        private PointLatLng N4LatLng;
        private PointLatLng N5LatLng;

        //Pixel Coordinates.
        private GPoint N1GPoint;
        private GPoint N2GPoint;
        private GPoint N3GPoint;
        private GPoint N4GPoint;
        private GPoint N5GPoint;

        private GMapControl MyFDGMapControl;

        public AnchorageTextOverlay(GMapControl FDGMapControl) //get parameter "FDGMapControl" for future use. but I'm not sure it is right way for programming.
        {
            MyFDGMapControl = FDGMapControl;
        }

        // Custom rendering logic here.
        // I put text overlay logic here.
        public override void OnRender(IGraphics g)
        {
            base.OnRender(g);

            N1LatLng = new PointLatLng(35.07187, 129.03975);
            N2LatLng = new PointLatLng(35.05754, 19.0270424);
            N3LatLng = new PointLatLng(35.05529, 129.05107);
            N4LatLng = new PointLatLng(35.03800, 129.06240);
            N5LatLng = new PointLatLng(35.01567, 129.06549);

            N1GPoint = Control.FromLatLngToLocal(N1LatLng);
            N2GPoint = Control.FromLatLngToLocal(N2LatLng);
            N3GPoint = Control.FromLatLngToLocal(N3LatLng);
            N4GPoint = Control.FromLatLngToLocal(N4LatLng);
            N5GPoint = Control.FromLatLngToLocal(N5LatLng);

            //Create color for font
            Color modifiedDarkGray = ControlPaint.DarkDark(Color.DarkGray); // get a darker version of DarkGray
            float brightness = 0.2f; // increase brightness by 20%
            float saturation = 0.3f; // saturation at 0 firstly(grayscale)
            float hue = 0.3f; // keep hue(color) at 0 (gray)
            float alpha = 1.0f; // keep alpha at 1 (opaque)
            modifiedDarkGray = FromAhsb(alpha, hue, saturation, brightness);

            // Add text above the polygons.
            using (Font font = new Font("Arial", 12))
            {
                using (SolidBrush textBrush = new SolidBrush(modifiedDarkGray)) 
                {
                    SizeF textSize = g.MeasureString("N1", font);
                    PointF textLocation = new PointF(N1GPoint.X, N1GPoint.Y);
                    g.DrawString("N1", font, textBrush, textLocation);

                    textSize = g.MeasureString("N2", font);
                    textLocation = new PointF(N2GPoint.X, N2GPoint.Y);
                    g.DrawString("N2", font, textBrush, textLocation);

                    textSize = g.MeasureString("N3", font);
                    textLocation = new PointF(N3GPoint.X, N3GPoint.Y);
                    g.DrawString("N3", font, textBrush, textLocation);

                    textSize = g.MeasureString("N4", font);
                    textLocation = new PointF(N4GPoint.X, N4GPoint.Y);
                    g.DrawString("N4", font, textBrush, textLocation);

                    textSize = g.MeasureString("N5", font);
                    textLocation = new PointF(N5GPoint.X, N5GPoint.Y);
                    g.DrawString("N5", font, textBrush, textLocation);
                }
            }
        }

        //functions that get color from Ahsb, for the font.
        private static Color FromAhsb(float alpha, float hue, float saturation, float brightness)
        {
            if (0 > alpha || 1 < alpha)
            {
                throw new ArgumentOutOfRangeException("alpha", alpha, "Value must be between 0 and 1.");
            }
            if (0 > hue || 360 < hue)
            {
                throw new ArgumentOutOfRangeException("hue", hue, "Value must be between 0 and 360.");
            }
            if (0 > saturation || 1 < saturation)
            {
                throw new ArgumentOutOfRangeException("saturation", saturation, "Value must be between 0 and 1.");
            }
            if (0 > brightness || 1 < brightness)
            {
                throw new ArgumentOutOfRangeException("brightness", brightness, "Value must be between 0 and 1.");
            }

            if (0 == saturation)
            {
                return Color.FromArgb((int)(alpha * 255), (int)(brightness * 255), (int)(brightness * 255), (int)(brightness * 255));
            }

            float fMax, fMid, fMin;
            int iSextant, iMax, iMid, iMin;

            if (0.5 < brightness)
            {
                fMax = brightness - (brightness * saturation) + saturation;
                fMin = brightness + (brightness * saturation) - saturation;
            }
            else
            {
                fMax = brightness + (brightness * saturation);
                fMin = brightness - (brightness * saturation);
            }

            iSextant = (int)Math.Floor(hue / 60f);
            if (300f <= hue)
            {
                hue -= 360f;
            }
            hue /= 60f;
            hue -= 2f * (float)Math.Floor(((iSextant + 1f) % 6f) / 2f);
            fMid = ((brightness - fMin) * hue) + fMin;
            iMax = (int)(fMax * 255);
            iMid = (int)(fMid * 255);
            iMin = (int)(fMin * 255);

            switch (iSextant)
            {
                case 0:
                    return Color.FromArgb((int)(alpha * 255), iMax, iMid, iMin);
                case 1:
                    return Color.FromArgb((int)(alpha * 255), iMid, iMax, iMin);
                case 2:
                    return Color.FromArgb((int)(alpha * 255), iMin, iMax, iMid);
                case 3:
                    return Color.FromArgb((int)(alpha * 255), iMin, iMid, iMax);
                case 4:
                    // Modified DarkGray color
                    int r = 85, g = 85, b = 85;
                    return Color.FromArgb((int)(alpha * 255), r, g, b);
                default:
                    return Color.FromArgb((int)(alpha * 255), iMax, iMin, iMid);
            }
        }
    }//end of class AnchorageTextOverlay
}// end of Namespace Regenerate Error Plugins